# coding: utf-8

"""
    Platform API

    _OpenAPI specification for the **Platform API** of **Helix**, the nomination platform of ElecLink_  ---  Additional documentation available in the API's [GitHub repository](https://github.com/eleclink-helix/platform-api) 

    The version of the OpenAPI document: 1.1.0
    Contact: team.eleclink@eleclink.co.uk
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from helix_platform_api_public_client.models.business_process_step_attempt import BusinessProcessStepAttempt

class TestBusinessProcessStepAttempt(unittest.TestCase):
    """BusinessProcessStepAttempt unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> BusinessProcessStepAttempt:
        """Test BusinessProcessStepAttempt
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `BusinessProcessStepAttempt`
        """
        model = BusinessProcessStepAttempt()
        if include_optional:
            return BusinessProcessStepAttempt(
                status = 'SCHEDULED',
                started_at = '2022-01-04T00:00:00.000Z',
                ended_at = '2022-01-04T00:00:00.000Z',
                fail_reason = ''
            )
        else:
            return BusinessProcessStepAttempt(
                status = 'SCHEDULED',
        )
        """

    def testBusinessProcessStepAttempt(self):
        """Test BusinessProcessStepAttempt"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()
