# coding: utf-8

"""
    Platform API

    _OpenAPI specification for the **Platform API** of **Helix**, the nomination platform of ElecLink_  ---  Additional documentation available in the API's [GitHub repository](https://github.com/eleclink-helix/platform-api) 

    The version of the OpenAPI document: 1.1.0
    Contact: team.eleclink@eleclink.co.uk
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class OrganisationAuthMethod(str, Enum):
    """
    OrganisationAuthMethod
    """

    """
    allowed enum values
    """
    PLATFORM = 'PLATFORM'
    SSO = 'SSO'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of OrganisationAuthMethod from a JSON string"""
        return cls(json.loads(json_str))


